'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createSSRContext = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint import/prefer-default-export: off */

var createSSRContext = exports.createSSRContext = function createSSRContext(settings) {
    var awaitStack = [];
    var callbacks = {};

    var rootUrl = settings.rootUrl ? settings.rootUrl : '/';

    var _apiUrl = settings.apiUrl ? settings.apiUrl : '/api';

    var checkTimer = null;

    // store local context to keep track of possible actions
    var ctx = {};

    var upsert = function upsert(action) {
        if (!callbacks[action]) {
            callbacks[action] = [];
        }
    };

    var emit = function emit(action) {
        upsert(action);
        callbacks[action].forEach(function (ticket) {
            if (ticket.once) {
                ticket.off();
            }
            ticket.fn();
        });
    };

    var checkStack = function checkStack() {
        if (awaitStack.length <= 0) {
            emit('complete');
        }
        return awaitStack.length;
    };

    var unsubscribe = function unsubscribe(action, ticket) {
        var idx = callbacks[action].indexOf(ticket);
        callbacks[action].splice(idx, 1);
    };

    var subscribe = function subscribe(action, fn) {
        var once = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        upsert(action);
        var ticket = { fn: fn, once: once };
        ticket.off = function () {
            return unsubscribe(action, ticket);
        };

        callbacks[action].push(ticket);
        setTimeout(checkStack);
        return ticket;
    };

    var reducer = (0, _extends3.default)({}, settings, {
        checkStack: checkStack,

        // event emitter
        on: function on(action, fn) {
            return subscribe(action, fn, false);
        },
        once: function once(action, fn) {
            return subscribe(action, fn, true);
        },

        // ssr compatible promise wrapper
        await: function _await(p) {
            awaitStack.push(p);

            p.then(function (v) {
                clearTimeout(checkTimer);
                var idx = awaitStack.indexOf(p);
                awaitStack.splice(idx, 1);
                checkTimer = setTimeout(checkStack);
            }, function (e) {
                clearTimeout(checkTimer);
                var idx = awaitStack.indexOf(p);
                awaitStack.splice(idx, 1);
                checkTimer = setTimeout(checkStack);
            });

            return p;
        },

        // perform a ssr compatible timeout
        setTimeout: function (_setTimeout) {
            function setTimeout(_x2, _x3) {
                return _setTimeout.apply(this, arguments);
            }

            setTimeout.toString = function () {
                return _setTimeout.toString();
            };

            return setTimeout;
        }(function (fn, delay) {
            return new Promise(function (resolve, reject) {
                setTimeout(function () {
                    try {
                        fn();
                        resolve();
                    } catch (err) {
                        reject(err);
                    }
                }, delay);
            });
        }),

        // allow for app related decision making
        isServer: function isServer() {
            return process.env.SSR === true;
        },
        isClient: function isClient() {
            return process.env.SSR !== true;
        },

        // deprecated
        apiUrl: function apiUrl(url) {
            console.warn('[create-ssr-context] apiUrl is deprecated, please use "getApiUrl()" instead');
            return ('' + _apiUrl + url).replace(/([^:]\/)\/+/g, '$1');
        },
        getRootUrl: function getRootUrl(url) {
            return ('' + rootUrl + url).replace(/([^:]\/)\/+/g, '$1');
        },
        getApiUrl: function getApiUrl(url) {
            return ('' + _apiUrl + url).replace(/([^:]\/)\/+/g, '$1');
        },
        getHistory: function getHistory() {
            return settings.history;
        },
        getRequestHandler: function getRequestHandler() {
            return settings.req;
        },
        getResponseHandler: function getResponseHandler() {
            return settings.res;
        },

        // redirect helper
        redirect: function redirect(to) {
            var code = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 300;

            ctx.redirect = to;
            ctx.redirectCode = code;
        },
        hasRedirect: function hasRedirect() {
            return ctx.redirect;
        },
        getRedirect: function getRedirect() {
            return ctx.redirect ? { url: ctx.redirect, code: ctx.redirectCode } : null;
        }
    });

    return {
        reducers: {
            ssr: function ssr() {
                return reducer;
            }
            // ssr: state => state || reducer,
        }
    };
};