'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createSSRRouter = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _createSsrRegisterHook = require('./create-ssr-register-hook');

var _serveAppStatic = require('./serve-app-static.middleware');

var _serveAppSsr = require('./serve-app-ssr.middleware');

var _serveBuild = require('./serve-build.middleware');

var _memcached = require('./memcached');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createSSRRouter = exports.createSSRRouter = function createSSRRouter() {
    var receivedSettings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var router = _express2.default.Router();

    var defaultSettings = {
        // do you want to run ssr or not?
        // @TODO: this might be a function so to make this decision dynamic
        enabled: String(process.env.REACT_APP_SSR_ENABLED || 'yes'),

        // comma serparated list of routes that will skip ssr
        blacklist: String(process.env.REACT_APP_SSR_BLACKLIST || ''),

        // react app's folders (absolute path)
        // we need to know where to locate your client's source files
        // and your ready-to-ship assets.
        source: String(process.env.REACT_APP_SSR_ROOT || _path2.default.join(process.cwd(), 'src')),
        build: String(process.env.REACT_APP_SSR_BUILD || _path2.default.join(process.cwd(), 'build')),

        // @TODO: find out a better way to handle callback urls client-to-server
        // durig ssr.
        port: String(process.env.REACT_APP_SSR_PORT || '8080'),

        // strips away all the javascript tags.
        // basically renders a pure HTML static web page.
        disableJs: String(process.env.REACT_APP_SSR_DISABLE_JS || 'no'),

        // maximum ssr execution time, useful to reduce server bottlenecks in case
        // apis or similar stuff get into the way
        renderTimeout: Number(process.env.REACT_APP_SSR_RENDER_TIMEOUT || 5000),

        // maximum number of ssr re-rendering waiting for asychronous code to finish
        // this is useful if the client enters in loops based on data loading that
        // does not preserve the results in between renderings
        // @TODO: document this with examples
        renderLimit: Number(process.env.REACT_APP_SSR_RENDER_LIMIT || 10),

        // CRA settings to handle images that might have beed embedded into the
        // shipped bundles, this is used by the "register-hook"
        embedExt: String(process.env.REACT_APP_SSR_EMBED_EXT || '.png,.jpg,jpeg,.gif,svg'),
        embedLimit: Number(process.env.REACT_APP_SSR_EMBED_LIMIT || 10000),

        // allow the in-memory cache system to kick in.
        useCache: String(process.env.REACT_APP_SSR_USE_CACHE || 'yes'),

        // by default the SSR middleware will send out the produced HTML to the client
        // but this could be avoided if the app will implement some custom
        // post-processing logic. 
        sendHtml: String(process.env.REACT_APP_SSR_SEND_HTML || 'yes'),

        // the HTML that is produced after a rendering is appended to the "req"
        // context variable for custom post processing.
        dataVar: String(process.env.REACT_APP_SSR_DATA_VAR || 'ssrOutput')
    };

    var settings = (0, _extends3.default)({}, defaultSettings, receivedSettings);

    // handle require images
    if (settings.enabled === 'yes') {
        (0, _memcached.init)(settings);
        (0, _createSsrRegisterHook.createSSRRegisterHook)(settings);
    }

    // serve client app
    var serveApp = settings.enabled === 'yes' ? _serveAppSsr.serveAppSSR : _serveAppStatic.serveAppStatic;

    // Optional blacklist out from ssr
    if (settings.enabled === 'yes' && settings.blacklist) {
        var rules = Array.isArray(settings.blacklist) ? settings.blacklist : settings.blacklist.split(',');
        rules.forEach(function (rule) {
            return router.get(rule, (0, _serveAppStatic.serveAppStatic)(settings));
        });
    }

    // Default routes handling
    router.get('/', serveApp(settings));
    router.use((0, _serveBuild.serveBuild)(settings));
    router.get('*', serveApp(settings));

    return router;
};