'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.serveAppSSR = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _reactHelmet = require('react-helmet');

var _mapModulesToBundles = require('./map-modules-to-bundles');

var _mapModulesToBundles2 = _interopRequireDefault(_mapModulesToBundles);

var _readSourceFile = require('./read-source-file');

var _readSourceFile2 = _interopRequireDefault(_readSourceFile);

var _prepHtml = require('./prep-html');

var _prepHtml2 = _interopRequireDefault(_prepHtml);

var _memcached = require('./memcached');

var cache = _interopRequireWildcard(_memcached);

var _currentVersion = require('./current-version');

var _currentVersion2 = _interopRequireDefault(_currentVersion);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Settings:
 * - root (string) - client app source folder absolute path
 * - build (string) - client app build folder absolute path
 * - port: (string) - ssr server port for default api calls
 * - renderTimeout: (int) - rendering timeout in milliseconds
 * - renderLimit: (int) - max amount of rerendering waiting for async calls to resolve
 * - disableJs: (string)[yes|no]
 * - nodeEnv: (string)[development|production]
 */
var staticRender = null;
var serveAppSSR = exports.serveAppSSR = function serveAppSSR() {
    var settings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return function () {
        var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(req, res, next) {
            var cachedHTML, filePath, htmlTemplate, initialState, prerender, helmet, mappedBundles, html;
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            if (!(settings.useCache === 'yes')) {
                                _context.next = 13;
                                break;
                            }

                            _context.prev = 1;
                            _context.next = 4;
                            return cache.get(req, res);

                        case 4:
                            cachedHTML = _context.sent;

                            if (!(typeof cachedHTML === 'string')) {
                                _context.next = 8;
                                break;
                            }

                            console.log('[ssr] (cached) ' + req.url);
                            return _context.abrupt('return', res.send(cachedHTML));

                        case 8:
                            _context.next = 13;
                            break;

                        case 10:
                            _context.prev = 10;
                            _context.t0 = _context['catch'](1);

                            console.log('[ssr] read cache error for "' + req.url + '" - ' + _context.t0.message);

                        case 13:
                            _context.prev = 13;

                            console.log('[ssr] ' + req.url);

                            // try to import the staticRender function from the app's source code

                            if (staticRender) {
                                _context.next = 29;
                                break;
                            }

                            _context.prev = 16;

                            staticRender = require(_path2.default.join(settings.source, 'app/ssr')).staticRender;

                            if (staticRender) {
                                _context.next = 20;
                                break;
                            }

                            throw new Error('ssr static render was not provided');

                        case 20:
                            if (!(typeof staticRender !== 'function')) {
                                _context.next = 22;
                                break;
                            }

                            throw new Error('ssr static render does not appear to be a function');

                        case 22:
                            if (!(staticRender.ssrVersion !== _currentVersion2.default)) {
                                _context.next = 24;
                                break;
                            }

                            throw new Error('ssr static render version signature does not appear to match');

                        case 24:
                            _context.next = 29;
                            break;

                        case 26:
                            _context.prev = 26;
                            _context.t1 = _context['catch'](16);

                            // fallback on a default static render that will visualize the error
                            staticRender = function staticRender() {
                                return {
                                    html: _context.t1.message,
                                    initialState: {},
                                    context: {}
                                };
                            };

                        case 29:
                            filePath = _path2.default.resolve(_path2.default.join(settings.build, 'index.html'));
                            _context.next = 32;
                            return (0, _readSourceFile2.default)(filePath);

                        case 32:
                            htmlTemplate = _context.sent;
                            initialState = {
                                app: {
                                    name: 'server'
                                },
                                ssr: {
                                    req: req, res: res, // proxy express handlers,
                                    //used to forward headers and cookies during ssr
                                    rootUrl: 'http://localhost:' + settings.port,
                                    apiUrl: 'http://localhost:' + settings.port + '/api'
                                }

                                // render app in order to get the initialState
                            };
                            _context.next = 36;
                            return staticRender(req.url, initialState, {
                                req: req, res: res, // proxy express handlers
                                context: {},
                                timeout: settings.renderTimeout, // max rendering duration
                                limit: settings.renderLimit // max re-rendering operations
                            });

                        case 36:
                            prerender = _context.sent;

                            if (!prerender.context.url) {
                                _context.next = 40;
                                break;
                            }

                            res.redirect(prerender.context.url);
                            return _context.abrupt('return');

                        case 40:

                            // get SEO headers for the page
                            helmet = _reactHelmet.Helmet.renderStatic();

                            // include dynamically requested bundles (via react-loadable)

                            _context.next = 43;
                            return (0, _mapModulesToBundles2.default)(settings, prerender.modules);

                        case 43:
                            mappedBundles = _context.sent;
                            html = (0, _prepHtml2.default)(htmlTemplate, (0, _extends3.default)({}, settings, {
                                html: helmet.htmlAttributes.toString(),
                                head: [helmet.title.toString(), helmet.meta.toString(), helmet.link.toString()].join(''),
                                body: prerender.html,
                                // need to remove req, res, history as they can cause circular
                                // shit in json stringify
                                state: (0, _extends3.default)({}, prerender.initialState, {
                                    ssr: (0, _extends3.default)({}, prerender.initialState.ssr, {
                                        req: null,
                                        res: null,
                                        history: null
                                    })
                                }),
                                jsBundles: mappedBundles.js,
                                cssBundles: mappedBundles.css
                            }));

                            // send out the page

                            if (settings.sendHtml === 'yes') {
                                res.send(html);
                            }

                            // store a cache version of the rendered html

                            if (!(settings.useCache === 'yes')) {
                                _context.next = 55;
                                break;
                            }

                            _context.prev = 47;
                            _context.next = 50;
                            return cache.set(req, res, html);

                        case 50:
                            _context.next = 55;
                            break;

                        case 52:
                            _context.prev = 52;
                            _context.t2 = _context['catch'](47);

                            console.log('[ssr] write cache error for "' + req.url + '" - ' + _context.t2.message);

                        case 55:

                            // decorate the request with the produced html so to allow
                            // the implementation of custom caching mechanism or post
                            // rendering filters
                            if (settings.sendHtml === 'no') {
                                req[settings.dataVar] = html;
                                next();
                            }
                            _context.next = 61;
                            break;

                        case 58:
                            _context.prev = 58;
                            _context.t3 = _context['catch'](13);

                            next(_context.t3);

                        case 61:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, undefined, [[1, 10], [13, 58], [16, 26], [47, 52]]);
        }));

        return function (_x2, _x3, _x4) {
            return _ref.apply(this, arguments);
        };
    }();
};